/*
 * failover -- The failover cmd tool.
 *
 * Copyright (C) 2009, OpenVox
 *
 * mark.liu <mark.liu@openvox.cn>
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License Version 2. See the LICENSE file
 * at the top of the source tree.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <termios.h>
#include <unistd.h>
#include "serial.h"

/* static char* get_baurd(speed_t baurd); */

static speed_struct_t baudrates[] = {
	{B50,		"50"}, {B75,		"75"}, {B110,		"110"},
	{B134,		"134"}, {B150,		"150"}, {B200,		"200"},
	{B300,		"300"}, {B600,		"600"}, {B1200,		"1200"},
	{B1800,		"1800"}, {B2400,	"2400"}, {B4800,	"4800"},
	{B9600,		"9600"}, {B19200,	"19200"}, {B38400,	"38400"},
	{B57600,	"57600"}, {B115200,	"115200"}, {0,		NULL}
};

/* get baurd */
speed_t parse_baurd(char *speed) {
	if (!speed) {
		return 0;
	}
	
	int i = 0;
	while ((baudrates[i].speedstr) && (strcmp(speed, baudrates[i].speedstr)))  {
		i++;
	}
	
	if (baudrates[i].speedstr) {
		return baudrates[i].speed;
	}
	
	return 0;
}

/*
static char* baurd2str(speed_t baurd) {
	if (!baurd) {
		return 0;
	}
	
	int i = 0;
	while ((baudrates[i].speed) && (baurd != baudrates[i].speed)) {
		i++;
	}
	
	if (baudrates[i].speed) {
		return baudrates[i].speedstr;
	}
	
	return NULL;
}
*/

int parse_parms(tcflag_t *c, char *parms, int flow, int modem) {
	if (strlen(parms) != 3) {
		fprintf(stderr, "Invalid parameter specification \"%s\"!\n", parms);
		return 1;
	}
	
	*c &= ~CSIZE;
	switch(parms[0]) {
	case '5':	*c |= CS5; break;
	case '6':	*c |= CS6; break;
	case '7':	*c |= CS7; break;
	case '8':	*c |= CS8; break;
	default:
		fprintf(stderr, "Invalid character size \"%c\": must be 5, 6, 7, or 8!\n",
			parms[0]);
		return 1;
	}
	
	switch(tolower(parms[1])) {
	case 'e':	*c |= PARENB; *c &= ~PARODD;	break;
	case 'n':	*c &= ~PARENB;					break;
	case 'o':	*c |= PARENB | PARODD;			break;
	default:
		fprintf(stderr, "Invalid parity \"%c\": must be E, N, or O!\n", parms[1]);
		return 1;
	}
	
	switch(parms[2]) {
	case '1':	*c &= ~CSTOPB;	break;
	case '2':	*c |= CSTOPB;	break;
	default:
		fprintf(stderr, "Invalid stop bit \"%c\": must be 1 or 2!\n", parms[2]);
		return 1;
	}
	
	*c &= ~CRTSCTS;
	if (flow)
		*c |= CRTSCTS;
	
	*c |= CLOCAL;
	if (modem)
		*c &= ~CLOCAL;
	
	return 0;
}


void print_parms(struct termios *ti, char *tty_name) {
	speed_t baudrate = 0;
	char bits, parity, stops;
	
	/* get speed */
	baudrate = cfgetispeed(ti);
	
	/* get bits */
	switch(ti->c_cflag & CSIZE) {
	case CS5: bits = '5'; break;
	case CS6: bits = '6'; break;
	case CS7: bits = '7'; break;
	case CS8: bits = '8'; break;
	default:
		bits ='?';
	}
	
	/* get parity */
	if (ti->c_cflag & PARENB) {
		parity = ti->c_cflag & PARODD ? 'O' : 'E';
	} else {
		parity = 'N';
	}
	
	/* get stops */
	stops = ti->c_cflag & CSTOPB ? '2' : '1';

	/*
	printf("Connected to %s at %s %c%c%c, modem status %s, %shardware handshake\n",
	tty_name, baurd2str(baudrate), bits, parity, stops, 
	ti->c_cflag & CLOCAL ? "ignored" : "observed",
	ti->c_cflag & CRTSCTS ? "" : "no ");
	*/
}


